@@ -2,6 +2,8 @@ import itertools
 from contextlib import contextmanager
 import enum
 import socket
+import sys
+from typing import TYPE_CHECKING
 
 import attr
 
@@ -25,6 +27,8 @@ from ._windows_cffi import (
     IoControlCodes,
 )
 
+assert not TYPE_CHECKING or sys.platform == "win32"
+
 # There's a lot to be said about the overall design of a Windows event
 # loop. See
 #
@@ -171,7 +175,8 @@ class CKeys(enum.IntEnum):
     AFD_POLL = 0
     WAIT_OVERLAPPED = 1
     LATE_CANCEL = 2
-    USER_DEFINED = 3  # and above
+    FORCE_WAKEUP = 3
+    USER_DEFINED = 4  # and above
 
 
 def _check(success):
@@ -180,7 +185,7 @@ def _check(success):
     return success
 
 
-def _get_base_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
+def _get_underlying_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
     if hasattr(sock, "fileno"):
         sock = sock.fileno()
     base_ptr = ffi.new("HANDLE *")
@@ -202,6 +207,53 @@ def _get_base_socket(sock, *, which=WSAIoctls.SIO_BASE_HANDLE):
     return base_ptr[0]
 
 
+def _get_base_socket(sock):
+    # There is a development kit for LSPs called Komodia Redirector.
+    # It does some unusual (some might say evil) things like intercepting
+    # SIO_BASE_HANDLE (fails) and SIO_BSP_HANDLE_SELECT (returns the same
+    # socket) in a misguided attempt to prevent bypassing it. It's been used
+    # in malware including the infamous Lenovo Superfish incident from 2015,
+    # but unfortunately is also used in some legitimate products such as
+    # parental control tools and Astrill VPN. Komodia happens to not
+    # block SIO_BSP_HANDLE_POLL, so we'll try SIO_BASE_HANDLE and fall back
+    # to SIO_BSP_HANDLE_POLL if it doesn't work.
+    # References:
+    # - https://github.com/piscisaureus/wepoll/blob/0598a791bf9cbbf480793d778930fc635b044980/wepoll.c#L2223
+    # - https://github.com/tokio-rs/mio/issues/1314
+
+    while True:
+        try:
+            # If this is not a Komodia-intercepted socket, we can just use
+            # SIO_BASE_HANDLE.
+            return _get_underlying_socket(sock)
+        except OSError as ex:
+            if ex.winerror == ErrorCodes.ERROR_NOT_SOCKET:
+                # SIO_BASE_HANDLE might fail even without LSP intervention,
+                # if we get something that's not a socket.
+                raise
+            if hasattr(sock, "fileno"):
+                sock = sock.fileno()
+            sock = _handle(sock)
+            next_sock = _get_underlying_socket(
+                sock, which=WSAIoctls.SIO_BSP_HANDLE_POLL
+            )
+            if next_sock == sock:
+                # If BSP_HANDLE_POLL returns the same socket we already had,
+                # then there's no layering going on and we need to fail
+                # to prevent an infinite loop.
+                raise RuntimeError(
+                    "Unexpected network configuration detected: "
+                    "SIO_BASE_HANDLE failed and SIO_BSP_HANDLE_POLL didn't "
+                    "return a different socket. Please file a bug at "
+                    "https://github.com/python-trio/trio/issues/new, "
+                    "and include the output of running: "
+                    "netsh winsock show catalog"
+                )
+            # Otherwise we've gotten at least one layer deeper, so
+            # loop back around to keep digging.
+            sock = next_sock
+
+
 def _afd_helper_handle():
     # The "AFD" driver is exposed at the NT path "\Device\Afd". We're using
     # the Win32 CreateFile, though, so we have to pass a Win32 path. \\.\ is
@@ -325,9 +377,7 @@ class WindowsIOManager:
         self._afd = None
 
         self._iocp = _check(
-            kernel32.CreateIoCompletionPort(
-                INVALID_HANDLE_VALUE, ffi.NULL, 0, 0
-            )
+            kernel32.CreateIoCompletionPort(INVALID_HANDLE_VALUE, ffi.NULL, 0, 0)
         )
         self._events = ffi.new("OVERLAPPED_ENTRY[]", MAX_EVENTS)
 
@@ -346,21 +396,41 @@ class WindowsIOManager:
         self._completion_key_counter = itertools.count(CKeys.USER_DEFINED)
 
         with socket.socket() as s:
-            # LSPs can't override this.
-            base_handle = _get_base_socket(s, which=WSAIoctls.SIO_BASE_HANDLE)
+            # We assume we're not working with any LSP that changes
+            # how select() is supposed to work. Validate this by
+            # ensuring that the result of SIO_BSP_HANDLE_SELECT (the
+            # LSP-hookable mechanism for "what should I use for
+            # select()?") matches that of SIO_BASE_HANDLE ("what is
+            # the real non-hooked underlying socket here?").
+            #
+            # This doesn't work for Komodia-based LSPs; see the comments
+            # in _get_base_socket() for details. But we have special
+            # logic for those, so we just skip this check if
+            # SIO_BASE_HANDLE fails.
+
             # LSPs can in theory override this, but we believe that it never
-            # actually happens in the wild.
-            select_handle = _get_base_socket(
+            # actually happens in the wild (except Komodia)
+            select_handle = _get_underlying_socket(
                 s, which=WSAIoctls.SIO_BSP_HANDLE_SELECT
             )
-            if base_handle != select_handle:  # pragma: no cover
-                raise RuntimeError(
-                    "Unexpected network configuration detected. "
-                    "Please file a bug at "
-                    "https://github.com/python-trio/trio/issues/new, "
-                    "and include the output of running: "
-                    "netsh winsock show catalog"
-                )
+            try:
+                # LSPs shouldn't override this...
+                base_handle = _get_underlying_socket(s, which=WSAIoctls.SIO_BASE_HANDLE)
+            except OSError:
+                # But Komodia-based LSPs do anyway, in a way that causes
+                # a failure with WSAEFAULT. We have special handling for
+                # them in _get_base_socket(). Make sure it works.
+                _get_base_socket(s)
+            else:
+                if base_handle != select_handle:
+                    raise RuntimeError(
+                        "Unexpected network configuration detected: "
+                        "SIO_BASE_HANDLE and SIO_BSP_HANDLE_SELECT differ. "
+                        "Please file a bug at "
+                        "https://github.com/python-trio/trio/issues/new, "
+                        "and include the output of running: "
+                        "netsh winsock show catalog"
+                    )
 
     def close(self):
         try:
@@ -392,7 +462,14 @@ class WindowsIOManager:
             completion_key_monitors=len(self._completion_key_queues),
         )
 
-    def handle_io(self, timeout):
+    def force_wakeup(self):
+        _check(
+            kernel32.PostQueuedCompletionStatus(
+                self._iocp, 0, CKeys.FORCE_WAKEUP, ffi.NULL
+            )
+        )
+
+    def get_events(self, timeout):
         received = ffi.new("PULONG")
         milliseconds = round(1000 * timeout)
         if timeout > 0 and milliseconds == 0:
@@ -400,15 +477,17 @@ class WindowsIOManager:
         try:
             _check(
                 kernel32.GetQueuedCompletionStatusEx(
-                    self._iocp, self._events, MAX_EVENTS, received,
-                    milliseconds, 0
+                    self._iocp, self._events, MAX_EVENTS, received, milliseconds, 0
                 )
             )
         except OSError as exc:
             if exc.winerror != ErrorCodes.WAIT_TIMEOUT:  # pragma: no cover
                 raise
-            return
-        for i in range(received[0]):
+            return 0
+        return received[0]
+
+    def process_events(self, received):
+        for i in range(received):
             entry = self._events[i]
             if entry.lpCompletionKey == CKeys.AFD_POLL:
                 lpo = entry.lpOverlapped
@@ -470,24 +549,21 @@ class WindowsIOManager:
                     # try changing this line to
                     # _core.reschedule(waiter, outcome.Error(exc))
                     raise exc
+            elif entry.lpCompletionKey == CKeys.FORCE_WAKEUP:
+                pass
             else:
                 # dispatch on lpCompletionKey
                 queue = self._completion_key_queues[entry.lpCompletionKey]
                 overlapped = int(ffi.cast("uintptr_t", entry.lpOverlapped))
                 transferred = entry.dwNumberOfBytesTransferred
                 info = CompletionKeyEventInfo(
-                    lpOverlapped=overlapped,
-                    dwNumberOfBytesTransferred=transferred,
+                    lpOverlapped=overlapped, dwNumberOfBytesTransferred=transferred
                 )
                 queue.put_nowait(info)
 
     def _register_with_iocp(self, handle, completion_key):
         handle = _handle(handle)
-        _check(
-            kernel32.CreateIoCompletionPort(
-                handle, self._iocp, completion_key, 0
-            )
-        )
+        _check(kernel32.CreateIoCompletionPort(handle, self._iocp, completion_key, 0))
         # Supposedly this makes things slightly faster, by disabling the
         # ability to do WaitForSingleObject(handle). We would never want to do
         # that anyway, so might as well get the extra speed (if any).
@@ -506,11 +582,7 @@ class WindowsIOManager:
         waiters = self._afd_waiters[base_handle]
         if waiters.current_op is not None:
             try:
-                _check(
-                    kernel32.CancelIoEx(
-                        self._afd, waiters.current_op.lpOverlapped
-                    )
-                )
+                _check(kernel32.CancelIoEx(self._afd, waiters.current_op.lpOverlapped))
             except OSError as exc:
                 if exc.winerror != ErrorCodes.ERROR_NOT_FOUND:
                     # I don't think this is possible, so if it happens let's
@@ -530,7 +602,7 @@ class WindowsIOManager:
             lpOverlapped = ffi.new("LPOVERLAPPED")
 
             poll_info = ffi.new("AFD_POLL_INFO *")
-            poll_info.Timeout = 2**63 - 1  # INT64_MAX
+            poll_info.Timeout = 2 ** 63 - 1  # INT64_MAX
             poll_info.NumberOfHandles = 1
             poll_info.Exclusive = 0
             poll_info.Handles[0].Handle = base_handle
@@ -669,9 +741,7 @@ class WindowsIOManager:
                     # We didn't request this cancellation, so assume
                     # it happened due to the underlying handle being
                     # closed before the operation could complete.
-                    raise _core.ClosedResourceError(
-                        "another task closed this resource"
-                    )
+                    raise _core.ClosedResourceError("another task closed this resource")
             else:
                 raise_winerror(code)
 
@@ -700,7 +770,7 @@ class WindowsIOManager:
             def submit_write(lpOverlapped):
                 # yes, these are the real documented names
                 offset_fields = lpOverlapped.DUMMYUNIONNAME.DUMMYSTRUCTNAME
-                offset_fields.Offset = file_offset & 0xffffffff
+                offset_fields.Offset = file_offset & 0xFFFFFFFF
                 offset_fields.OffsetHigh = file_offset >> 32
                 _check(
                     kernel32.WriteFile(
@@ -722,7 +792,7 @@ class WindowsIOManager:
 
             def submit_read(lpOverlapped):
                 offset_fields = lpOverlapped.DUMMYUNIONNAME.DUMMYSTRUCTNAME
-                offset_fields.Offset = file_offset & 0xffffffff
+                offset_fields.Offset = file_offset & 0xFFFFFFFF
                 offset_fields.OffsetHigh = file_offset >> 32
                 _check(
                     kernel32.ReadFile(
