@@ -1,3 +1,5 @@
+# coding: utf-8
+
 # Little utilities we use internally
 
 from abc import ABCMeta
@@ -196,9 +198,7 @@ class ConflictDetector:
 
 
 def async_wraps(cls, wrapped_cls, attr_name):
-    """Similar to wraps, but for async wrappers of non-async functions.
-
-    """
+    """Similar to wraps, but for async wrappers of non-async functions."""
 
     def decorator(func):
         func.__name__ = attr_name
@@ -279,7 +279,7 @@ class generic_function:
 # inherit from these metaclasses. Fortunately, GenericMeta inherits from
 # ABCMeta, so inheriting from GenericMeta alone is sufficient (when it
 # exists at all).
-if hasattr(t, "GenericMeta"):
+if not t.TYPE_CHECKING and hasattr(t, "GenericMeta"):
     BaseMeta = t.GenericMeta
 else:
     BaseMeta = ABCMeta
@@ -323,6 +323,9 @@ class SubclassingDeprecatedIn_v0_15_0(BaseMeta):
         return super().__new__(cls, name, bases, cls_namespace)
 
 
+T = t.TypeVar("T")
+
+
 class NoPublicConstructor(Final):
     """Metaclass that enforces a class to be final (i.e., subclass not allowed)
     and ensures a private constructor.
@@ -342,10 +345,27 @@ class NoPublicConstructor(Final):
     - TypeError if a sub class or an instance is created.
     """
 
-    def __call__(self, *args, **kwargs):
+    def __call__(cls, *args, **kwargs):
         raise TypeError(
-            f"{self.__module__}.{self.__qualname__} has no public constructor"
+            f"{cls.__module__}.{cls.__qualname__} has no public constructor"
         )
 
-    def _create(self, *args, **kwargs):
-        return super().__call__(*args, **kwargs)
+    def _create(cls: t.Type[T], *args: t.Any, **kwargs: t.Any) -> T:
+        return super().__call__(*args, **kwargs)  # type: ignore
+
+
+def name_asyncgen(agen):
+    """Return the fully-qualified name of the async generator function
+    that produced the async generator iterator *agen*.
+    """
+    if not hasattr(agen, "ag_code"):  # pragma: no cover
+        return repr(agen)
+    try:
+        module = agen.ag_frame.f_globals["__name__"]
+    except (AttributeError, KeyError):
+        module = "<{}>".format(agen.ag_code.co_filename)
+    try:
+        qualname = agen.__qualname__
+    except AttributeError:
+        qualname = agen.ag_code.co_name
+    return f"{module}.{qualname}"
