@@ -149,7 +149,7 @@ async def ssl_echo_server_raw(**kwargs):
         # nursery context manager to exit too.
         with a, b:
             nursery.start_soon(
-                trio.to_thread.run_sync, partial(ssl_echo_serve_sync, b, **kwargs),
+                trio.to_thread.run_sync, partial(ssl_echo_serve_sync, b, **kwargs)
             )
 
             yield SocketStream(tsocket.from_stdlib_socket(a))
@@ -231,7 +231,7 @@ class PyOpenSSLEchoStream:
         return self._conn.renegotiate_pending()
 
     def renegotiate(self):
-        # Returns false if a renegotation is already in progress, meaning
+        # Returns false if a renegotiation is already in progress, meaning
         # nothing happens.
         assert self._conn.renegotiate()
 
@@ -323,7 +323,7 @@ class PyOpenSSLEchoStream:
 async def test_PyOpenSSLEchoStream_gives_resource_busy_errors():
     # Make sure that PyOpenSSLEchoStream complains if two tasks call send_all
     # at the same time, or ditto for receive_some. The tricky cases where SSLStream
-    # might accidentally do this are during renegotation, which we test using
+    # might accidentally do this are during renegotiation, which we test using
     # PyOpenSSLEchoStream, so this makes sure that if we do have a bug then
     # PyOpenSSLEchoStream will notice and complain.
 
@@ -1245,7 +1245,7 @@ async def test_SSLListener(client_ctx):
 
         transport_client = await open_tcp_stream(*listen_sock.getsockname())
         ssl_client = SSLStream(
-            transport_client, client_ctx, server_hostname="trio-test-1.example.org",
+            transport_client, client_ctx, server_hostname="trio-test-1.example.org"
         )
         return listen_sock, ssl_listener, ssl_client
 
